#include "CC2530_ADC.h"



static uint8 adcRef;

/**************************************************************************************************
 *                                            CONSTANTS
 **************************************************************************************************/
#define HAL_ADC_EOC         0x80    /* End of Conversion bit */
#define HAL_ADC_REF_AVDD          0x80    /* AVDD_SOC Pin Reference */



/**************************************************************************************************
 * @fn      HalAdcInit
 *
 * @brief   Initialize ADC Service
 *
 * @param   None
 *
 * @return  None
 **************************************************************************************************/
void HalAdcInit (void)
{

  adcRef = HAL_ADC_REF_AVDD;
}

/**************************************************************************************************
 * @fn      HalAdcRead
 *
 * @brief   Read the ADC based on given channel and resolution
 *
 * @param   channel - channel where ADC will be read
 * @param   resolution - the resolution of the value
 *
 * @return  16 bit value of the ADC in offset binary format.
 *
 *          Note that the ADC is "bipolar", which means the GND (0V) level is mid-scale.
 *          Note2: This function assumes that ADCCON3 contains the voltage reference.
 **************************************************************************************************/

uint16 HalAdcRead (uint8 channel, uint8 resolution)
{

  uint16  reading = 0;
  uint8   i, resbits;
  uint8  adcChannel = 1;
  if (channel < 8)
  {
    for (i=0; i < channel; i++)
    {
      adcChannel <<= 1;
    }
  }

  ADCCFG |= adcChannel;

  switch (resolution)//7,9,10,12
  {
    case 7:
      resbits = 0x00;
      break;
    case 9:
      resbits = 0x10;
      break;
    case 10:
      resbits = 0x20;
      break;
    case 12:
      resbits = 0x30;
      break;
    default:
      resbits =0x30;
      break;
  }

  /* writing to this register starts the extra conversion */
  ADCCON3 = channel | resbits | adcRef;

  /* Wait for the conversion to be done */
  while (!(ADCCON1 & HAL_ADC_EOC));

  /* Disable channel after done conversion */
  ADCCFG &= (adcChannel ^ 0xFF);

  /* Read the result */
  reading = (uint16) (ADCL);
  reading |= (uint16) (ADCH << 8);

  /* Treat small negative as 0 */
  if (reading < 0)
    reading = 0;
  switch (resolution)
  {
    case 7:
      reading >>= 9;
      break;
    case 9:
      reading >>= 7;
      break;
    case 10:
      reading >>= 6;
      break;
    case 12:
    default:
      reading >>= 4;
    break;
  }
  return ((uint16)reading);
}